@@ -2,7 +2,6 @@ package Jifty::Dispatcher;
 use strict;
 use warnings;
 use Exporter;
-use Jifty::YAML;
 use base qw/Exporter Jifty::Object/;
            
 
@@ -12,6 +11,11 @@ Jifty::Dispatcher - The Jifty Dispatcher
 
 =head1 SYNOPSIS
 
+In your F<autohandler>, put these two lines first:
+
+    require MyApp::Dispatcher;
+    MyApp::Dispatcher->handle_request;
+
 In B<MyApp::Dispatcher>:
 
     package MyApp::Dispatcher;
@@ -67,13 +71,13 @@ The Dispatcher runs rules in several stages:
 
 B<before> rules are run before Jifty evaluates actions. They're the
 perfect place to enable or disable L<Jifty::Action>s using
-L<Jifty::API/allow> and L<Jifty::API/deny> or to completely disallow
-user access to private I<component> templates such as the F<_elements>
-directory in a default Jifty application.  They're also the right way
-to enable L<Jifty::LetMe> actions.
+L<Jifty::Web/allow_actions> and L<Jifty::Web/deny_actions> or to
+completely disallow user access to private I<component> templates such
+as the F<_elements> directory in a default Jifty application.  They're
+also the right way to enable L<Jifty::LetMe> actions.
 
-You can entirely stop processing with the C<redirect>, C<tangent> and
-C<abort> directives, though L</after> rules will still run.
+You can entirely stop processing with the C<redirect> and C<abort>
+directives.
 
 =item on
 
@@ -83,15 +87,15 @@ place to set up view-specific objects or load up values for your
 templates.
 
 Dispatcher directives are evaluated in order until we get to either a
-C<show>, C<redirect>, C<tangent> or C<abort>.
+C<show>, C<redirect> or an C<abort>.
 
 =item after
 
 L<after> rules let you clean up after rendering your page. Delete your
 cache files, write your transaction logs, whatever.
 
-At this point, it's too late to C<show>, C<redirect>, C<tangent> or C<abort>
-page display.
+At this point, it's too late to C<show>, C<redirect> or C<abort> page
+display.
 
 =back
 
@@ -105,34 +109,6 @@ common C<$Dispatcher> object.
 
 =cut
 
-=head1 Plugins and rule ordering
-
-By default, L<Jifty::Plugin> dispatcher rules are added in the order
-they are specified in the application's configuration file; that is,
-after all the plugin dispatchers have run in order, then the
-application's dispatcher runs.  It is possible to specify rules which
-should be reordered with respect to this rule, however.  This is done
-by using a variant on the C<before> and C<after> syntax:
-
-    before plugin NAME =>
-        RULE(S);
-    
-    after plugin NAME =>
-        RULE(S);
-
-C<NAME> may either be a string, which must match the plugin name
-exactly, or a regular expression, which is matched against the plugin
-name.  The rule will be placed at the first boundary that it matches --
-that is, given a C<before plugin qr/^Jifty::Plugin::Auth::/> and both
-a C<Jifty::Plugin::Auth::Basic> and a C<Jifty::Plugin::Auth::Complex>,
-the rules will be placed before the first.
-
-C<RULES> may either be a single C<before>, C<on>, C<under>, or
-C<after> rule to change the ordering of, or an array reference of
-rules to reorder.
-
-=cut
-
 =head1 Data your dispatch routines has access to
 
 =head2 request
@@ -220,11 +196,11 @@ Deletes an argument we were passing to our template.
 =head2 show $component
 
 Display the presentation component.  If not specified, use the
-request path as the default page.
+default page in call_next.
 
 =head2 dispatch $path
 
-Dispatch again using $path as the request path, preserving args.
+eispatch again using $path as the request path, preserving args.
 
 =head2 next_rule
 
@@ -236,21 +212,16 @@ Break out from the current C<run> block and stop running rules in this stage.
 
 =head2 abort $code
 
-Abort the request; this skips straight to the cleanup stage.
-
-If C<$code> is specified, it's used as the HTTP status code.
+Abort the request.
 
 =head2 redirect $uri
 
 Redirect to another URI.
 
-=head2 tangent $uri
 
-Take a continuation here, and tangent to another URI.
+=head2 next_show
 
-=head2 plugin
-
-See L</Plugins and rule ordering>, above.
+INTERNAL MAGIC YOU SHOULD NOT USE THAT ALEX SHOULD RENAME ;)
 
 =cut
 
@@ -259,16 +230,12 @@ our @EXPORT = qw<
 
     before on after
 
-    show dispatch abort redirect tangent
+    show dispatch abort redirect
 
     GET POST PUT HEAD DELETE OPTIONS
 
-    plugin
-
     get next_rule last_rule
 
-    already_run
-
     $Dispatcher
 >;
 
@@ -285,7 +252,6 @@ sub run (&@)      { _ret @_ }    # execute a block of code
 sub show (;$@)    { _ret @_ }    # render a page
 sub dispatch ($@) { _ret @_ }    # run dispatch again with another URI
 sub redirect ($@) { _ret @_ }    # web redirect
-sub tangent ($@)  { _ret @_ }    # web tangent
 sub abort (;$@)   { _ret @_ }    # abort request
 sub default ($$@) { _ret @_ }    # set parameter if it's not yet set
 sub set ($$@)     { _ret @_ }    # set parameter
@@ -300,8 +266,6 @@ sub HEAD ($)    { _qualify method => @_ }
 sub DELETE ($)  { _qualify method => @_ }
 sub OPTIONS ($) { _qualify method => @_ }
 
-sub plugin ($) { return { plugin => @_ } }
-
 =head2 import
 
 Jifty::Dispatcher is an L<Exporter>, that is, part of its role is to
@@ -322,8 +286,7 @@ sub import {
     my @args  = grep { !/^-[Bb]ase/ } @_;
 
     no strict 'refs';
-    no warnings 'once';
-    for (qw(RULES_RUN RULES_SETUP RULES_CLEANUP RULES_DEFERRED)) {
+    for (qw(RULES RULES_SETUP RULES_CLEANUP)) {
         @{ $pkg . '::' . $_ } = ();
     }
     if ( @args != @_ ) {
@@ -359,26 +322,21 @@ sub _ret (@) {
     } elsif ( defined wantarray ) {
         [ [ $op => splice( @_, 0, length($proto) ) ], @_ ];
     } else {
-        _push_rule($pkg, [ $op => splice( @_, 0, length($proto) ) ] );
-    }
-}
+        my $ruleset;
+        if ( $op eq 'before' ) {
+            $ruleset = 'RULES_SETUP';
+        } elsif ( $op eq 'after' ) {
+            $ruleset = 'RULES_CLEANUP';
+        } else {
+            $ruleset = 'RULES_RUN';
+        }
 
-sub _push_rule($$) {
-    my($pkg, $rule) = @_;
-    my $op = $rule->[0];
-    my $ruleset;
-    if ( ($op eq "before" or $op eq "after") and ref $rule->[1] and ref $rule->[1] eq 'HASH' and $rule->[1]{plugin} ) {
-        $ruleset = 'RULES_DEFERRED';
-    } elsif ( $op eq 'before' ) {
-        $ruleset = 'RULES_SETUP';
-    } elsif ( $op eq 'after' ) {
-        $ruleset = 'RULES_CLEANUP';
-    } else {
-        $ruleset = 'RULES_RUN';
+        no strict 'refs';
+
+        # XXX TODO, need to spec stage here.
+        push @{ $pkg . '::' . $ruleset },
+            [ $op => splice( @_, 0, length($proto) ) ], @_;
     }
-    no strict 'refs';
-    # XXX TODO, need to spec stage here.
-    push @{ $pkg . '::' . $ruleset }, $rule;
 }
 
 sub _qualify ($@) {
@@ -448,47 +406,17 @@ to do is to put the following two lines first:
 sub handle_request {
     my $self = shift;
 
+    my $path = Jifty->web->request->path;
+    # XXX TODO: jesse commented this out because it weirdly breaks things
+    #    $path =~ s{/index\.html$}{};
+
     local $Dispatcher = $self->new();
 
-    # XXX TODO: refactor this out somehow?
-    # We don't want the previous mason request hanging aroudn once we start dispatching
-    local $HTML::Mason::Commands::m = undef;
-    # Mason introduces a DIE handler that generates a mason exception
-    # which in turn generates a backtrace. That's fine when you only
-    # do it once per request. But it's really, really painful when you
-    # do it often, as is the case with fragments
-    local $SIG{__DIE__} = 'DEFAULT';
-
-    eval {
-        $Dispatcher->_do_dispatch( Jifty->web->request->path);
-    };
-    if ( my $err = $@ ) {
-        $self->log->warn(ref($err) . " " ."'$err'") if ( $err !~ /^ABORT/ );
+HANDLER: {
+        $Dispatcher->_do_dispatch($path);
     }
 }
 
-=head2 _handle_stage NAME, EXTRA_RULES
-
-Handles the all rules in the stage named C<NAME>.  Additionally, any
-other arguments passed after the stage C<NAME> are added to the end of
-the rules for that stage.
-
-This is the unit which calling L</last_rule> skips to the end of.
-
-=cut
-
-sub _handle_stage {
-    my ($self, $stage, @rules) = @_;
-
-    eval { $self->_handle_rules( [ $self->rules($stage), @rules ] ); };
-    if ( my $err = $@ ) {
-        $self->log->warn( ref($err) . " " . "'$err'" )
-            if ( $err !~ /^(LAST RULE|ABORT)/ );
-        return $err =~ /^ABORT/ ? 0 : 1;
-    }
-    return 1;
-}
-
 =head2 _handle_rules RULESET
 
 When handed an arrayref or array of rules (RULESET), walks through the 
@@ -524,17 +452,18 @@ sub _handle_rule {
     my ( $self, $rule ) = @_;
     my ( $op,   @args );
 
-    # Handle the case where $rule is an array reference.
-    if (ref($rule) eq 'ARRAY') {
-        ( $op, @args ) = @$rule;
-    } else {
-        ( $op, @args ) = ( run => $rule );
+    # Handle the case where $op is a code reference.
+    {
+        local $@;
+        eval { ( $op, @args ) = @$rule };
+        ( $op, @args ) = ( run => $rule ) if $@;
     }
 
     # Handle the case where $op is an array.
     my $sub_rules;
-    if (ref($op) eq 'ARRAY' ) {
-         $sub_rules = [ @$op, @args ];
+    {
+        local $@;
+        eval { $sub_rules = [ @$op, @args ] };
     }
 
     if ($sub_rules) {
@@ -553,7 +482,8 @@ sub _handle_rule {
 no warnings 'exiting';
 
 sub next_rule { next RULE }
-sub last_rule { die "LAST RULE" }
+sub last_rule { die "LAST RULE"; }
+sub next_show { last HANDLE_WEB }
 
 =head2 _do_under
 
@@ -633,41 +563,16 @@ This method is called by the dispatcher internally. You shouldn't need to.
 sub _do_after {
     my ( $self, $cond, $rules ) = @_;
     if ( my $regex = $self->_match($cond) ) {
-        $self->log->debug("Matched 'after' rule $regex for ".$self->{'path'});
+        $self->log->debug("Matched 'on' rule $regex for ".$self->{'path'});
         # match again to establish $1 $2 etc in the dynamic scope
         $self->{path} =~ $regex;
         $self->_handle_rules($rules);
     }
 }
 
-=head2 already_run
-
-Returns true if the code block has run once already in this request.
-This can be useful for 'after' rules to ensure that they only run
-once, even if there is a sub-dispatch which would cause it to run more
-than once.  The idiom is:
-
-    after '/some/path/*' => run {
-        return if already_run;
-        # ...
-    };
-
-=cut
-
-sub already_run {
-    my $id = $Dispatcher->{call_rule};
-    return 1 if get "__seen_$id";
-    set "__seen_$id" => 1;
-    return 0;
-}
-
 sub _do_run {
     my ( $self, $code ) = @_;
 
-    # Keep track of the coderef being run, so we can know about
-    # already_run
-    local $self->{call_rule} = $code;
-
     # establish void context and make a call
     ( $self->can($code) || $code )->();
 
@@ -681,7 +586,7 @@ sub _do_run {
 
 This method is called by the dispatcher internally. You shouldn't need to.
 
-Redirect the user to the URL provided in the mandatory PATH argument.
+Redirect the user to the URL provded in the mandatory PATH argument.
 
 =cut
 
@@ -689,21 +594,7 @@ sub _do_redirect {
     my ( $self, $path ) = @_;
     $self->log->debug("Redirecting to $path");
     Jifty->web->redirect($path);
-}
-
-=head2 _do_tangent PATH
-
-This method is called by the dispatcher internally. You shouldn't need to.
-
-Take a tangent to the URL provided in the mandatory PATH argument.
-(See L<Jifty::Manual::Continuation> for more about tangents.)
-
-=cut
-
-sub _do_tangent {
-    my ( $self, $path ) = @_;
-    $self->log->debug("Taking a tangent to $path");
-    Jifty->web->tangent(url => $path);
+    last_rule;
 }
 
 =head2 _do_abort 
@@ -717,21 +608,9 @@ Don't display any page. just stop.
 sub _do_abort {
     my $self = shift;
     $self->log->debug("Aborting processing");
-    if (@_) {
-        # This is the status code
-        my $status = shift;
-        my $apache = Jifty->handler->apache;
-        $apache->header_out(Status => $status);
-        $apache->send_http_header;
-
-        require HTTP::Status;
-        print STDOUT $status, ' ' , HTTP::Status::status_message($status);
-    }
-    $self->_abort;
+    last_rule;
 }
 
-sub _abort { die "ABORT" }
-
 =head2 _do_show [PATH]
 
 This method is called by the dispatcher internally. You shouldn't need to.
@@ -747,31 +626,38 @@ sub _do_show {
 
     # Fix up the path
     $path = shift if (@_);
-    $path ||= $self->{path};
-    $self->log->debug("Showing path $path");
+    $path ||= request->path;
 
-    # If we've got a working directory (from an "under" rule) and we have
+    # If we've got a working directory (from an "under" rule) and we have 
     # a relative path, prepend the working directory
     $path = "$self->{cwd}/$path" unless $path =~ m{^/};
 
-    # When we're requesting a directory, go looking for the index.html
-    if ( $self->template_exists( $path . "/index.html" ) ) {
-        $path .= "/index.html";
+    # If we're requesting a directory, go looking for the index.html
+    if ($path =~ m{/$} and 
+        Jifty->handler->mason->interp->comp_exists($path."/index.html")) {
+         $path .= "index.html" 
     }
+    # Redirect to directory (and then index) if they requested 
+    # the directory itself
+    
+    # XXX TODO, we should search all component roots
+    $self->_do_redirect($path . "/") 
+      if -d Jifty::Util->absolute_path( (Jifty->config->framework('Web')->{'TemplateRoot'} || "html") . $path );
+    
+    # Set the request path
+    request->path($path);
 
-    my $abs_template_path = Jifty::Util->absolute_path(
-        Jifty->config->framework('Web')->{'TemplateRoot'} . $path );
-    my $abs_root_path = Jifty::Util->absolute_path(
-        Jifty->config->framework('Web')->{'TemplateRoot'} );
-
-    if ( $abs_template_path !~ /^\Q$abs_root_path\E/ ) {
-        request->path('/__jifty/errors/500');
-    } else {
-        # Set the request path
-        request->path($path);
+    $self->log->debug("Having Mason handle ".request->path);
+    eval { Jifty->handler->mason->handle_comp(request->path); };
+    my $err = $@;
+    # Handle parse errors
+    if ( $err and not UNIVERSAL::isa $err, 'HTML::Mason::Exception::Abort' ) {
+        # XXX TODO: get this into the browser somehow
+        warn "Mason error: $err";
+        Jifty->web->redirect("/__jifty/error/mason_internal_error");
+    } elsif ($err) {
+        die $err;
     }
-    $self->render_template( request->path );
-
     last_rule;
 }
 
@@ -809,30 +695,27 @@ Once it's done with that, it runs all the cleanup rules defined with C<after>.
 sub _do_dispatch {
     my $self = shift;
 
-    # Requests should always start with a leading /
-    $self->{path} = "/".shift;
+    $self->{path} = shift;
     $self->{cwd}  = '';
 
     # Normalize the path.
     $self->{path} =~ s{/+}{/}g;
+    $self->{path} =~ s{/$}{};
 
     $self->log->debug("Dispatching request to ".$self->{path});
-
-    # Setup -- we we don't abort out of setup, then run the
-    # actions and then the RUN stage.
-    if ($self->_handle_stage('SETUP')) {
-        # Run actions
-        Jifty->web->handle_request();
-
-        # Run, and show the page
-        $self->_handle_stage('RUN' => 'show');
+    eval {
+        HANDLER: {
+            $self->_handle_rules( [ $self->rules('SETUP') ] );
+          HANDLE_WEB: {
+                Jifty->web->handle_request();
+            }
+            $self->_handle_rules( [ $self->rules('RUN'), 'show' ] );
+            $self->_handle_rules( [ $self->rules('CLEANUP') ] );
+        }
+    };
+    if ( my $err = $@ ) {
+        warn ref($err) . " " ."'$err'" if ( $err !~ /^LAST RULE/);
     }
-
-    # Cleanup
-    $self->_handle_stage('CLEANUP');
-
-    # Out to the next dispatcher's cleanup
-    $self->_abort;
 }
 
 =head2 _match CONDITION
@@ -886,10 +769,7 @@ sub _match {
             }
 
             # All precondition passed, get original condition literal
-            return $self->_match( $cond->{''} ) if $cond->{''};
-
-            # Or, if we don't have a literal, we win.
-            return 1;
+            return $self->_match( $cond->{''} );
         };
         if ( my $err = $@ ) {
             warn "$self _match failed: $err";
@@ -915,19 +795,13 @@ came in with that method.
 
 sub _match_method {
     my ( $self, $method ) = @_;
-    $self->log->debug("Matching URL $ENV{REQUEST_METHOD} against ".$method);
-    lc( $ENV{REQUEST_METHOD} ) eq lc($method);
-}
-
-sub _match_plugin {
-    my ( $self, $plugin ) = @_;
-    warn "Deferred check shouldn't happen";
-    return 0;
+    $self->log->debug("Matching URL ".$self->{cgi}->method." against ".$method);
+    lc( $self->{cgi}->method ) eq lc($method);
 }
 
 =head2 _compile_condition CONDITION
 
-Takes a condition defined as a simple string and return it as a regex
+Takes a condition defined as a simple string ad return it as a regex
 condition.
 
 =cut
@@ -943,11 +817,6 @@ sub _compile_condition {
     $cond =~ s{(?:\\\/)+}{/}g;
     $cond =~ s{/$}{};
 
-    my $has_capture = ( $cond =~ / \\ [*?] /x);
-    if ($has_capture or $cond =~ / \\ [[{] /x) {
-        $cond = $self->_compile_glob($cond);
-    }
-
     if ( $cond =~ m{^/} ) {
 
         # '/foo' => qr{^/foo}
@@ -961,11 +830,10 @@ sub _compile_condition {
         # empty path -- just match $cwd itself
         $cond = "(?<=\\A$self->{cwd})";
     }
-
     if ( $Dispatcher->{rule} eq 'on' ) {
 
         # "on" anchors on complete match only
-        $cond .= '/?\\z';
+        $cond .= '\\z';
     } else {
 
         # "in" anchors on prefix match in directory boundary
@@ -973,7 +841,9 @@ sub _compile_condition {
     }
 
     # Make all metachars into capturing submatches
-    if (!$has_capture) {
+    unless ( $cond
+        =~ s{( (?: \\ [*?] )+ )}{'('. $self->_compile_glob($1) .')'}egx )
+    {
         $cond = "($cond)";
     }
 
@@ -984,307 +854,18 @@ sub _compile_condition {
 
 Private function.
 
-Turns a metaexpression containing C<*> and C<?> into a capturing regex pattern.
-
-Also supports the non-capturing C<[]> and C<{}> notation.
-
-The rules are:
-
-=over 4
-
-=item *
-
-A C<*> between two C</> characters, or between a C</> and end of string,
-should match one or more non-slash characters:
-
-    /foo/*/bar
-    /foo/*/
-    /foo/*
-    /*
-
-=item *
-
-All other C<*> can match zero or more non-slash characters: 
-
-    /*bar
-    /foo*bar
-    *
-
-=item *
-
-Two stars (C<**>) can match zero or more characters, including slash:
-
-    /**/bar
-    /foo/**
-    **
-
-=item *
-
-Consecutive C<?> marks are captured together:
-
-    /foo???bar      # One capture for ???
-    /foo??*         # Two captures, one for ?? and one for *
-
-=item *
-
-Brackets such as C<[a-z]> denote character classes; they are not captured.
-
-=item *
-
-Braces such as C<{xxx,yyy}]> denote alternations; they are not captured.
-
-=back
+Turns a metaexpression containing * and ? into a capturing perl regex pattern.
 
 =cut
 
 sub _compile_glob {
     my ( $self, $glob ) = @_;
-    $glob =~ s{
-        # Stars between two slashes, or between a slash and end-of-string,
-        # should at match one or more non-slash characters.
-        (?<= /)      # lookbehind for slash
-        \\ \*        # star
-        (?= / | \z)  # lookahead for slash or end-of-string
-    }{([^/]+)}gx;
-    $glob =~ s{
-        # Two stars can match zero or more characters, including slash.
-        \\ \* \\ \*
-    }{(.*)}gx;
-    $glob =~ s{
-        # All other stars can match zero or more non-slash character.
-        \\ \*
-    }{([^/]*)}gx;
-    $glob =~ s{
-        # Consecutive question marks are captured as one unit;
-        # we do this by capturing them and then repeat the result pattern
-        # for that many times.  The divide-by-two takes care of the
-        # extra backslashes.
-        ( (?: \\ \? )+ )
-    }{([^/]{${ \( length($1)/2 ) }})}gx;
-    $glob =~ s{
-        # Brackets denote character classes
-        (
-            \\ \[           # opening
-            (?:             # one or more characters:
-                \\ \\ \\ \] # ...escaped closing bracket
-            |
-                \\ [^\]]    # ...escaped (but not the closing bracket)
-            |
-                [^\\]       # ...normal
-            )+
-            \\ \]           # closing
-        )
-    }{$self->_unescape($1)}egx;
-    $glob =~ s{
-        # Braces denote alternations
-        \\ \{ (         # opening (not part of expression)
-            (?:             # one or more characters:
-                \\ \\ \\ \} # ...escaped closing brace
-            |
-                \\ [^\}]    # ...escaped (but not the closing brace)
-            |
-                [^\\]       # ...normal
-            )+
-        ) \\ \}         # closing (not part of expression)
-    }{'(?:'.join('|', split(/\\,/, $1)).')'}egx;
+    $glob =~ s{\\}{}g;
+    $glob =~ s{\*}{[^/]+}g;
+    $glob =~ s{\?}{[^/]}g;
     $glob;
 }
 
-sub _unescape {
-    my $self = shift;
-    my $text = shift;
-    $text =~ s{\\(.)}{$1}g;
-    return $text;
-}
 
-=head2 template_exists PATH
-
-Returns true if PATH is a valid template inside your template root.
-
-=cut
-
-sub template_exists {
-    my $self = shift;
-    my $template = shift;
-
-    return  Jifty->handler->mason->interp->comp_exists( $template);
-}
-
-
-=head2 render_template PATH
-
-Use our templating system to render a template. If there's an error, do the right thing.
-
-
-=cut
-
-sub render_template {
-    my $self = shift;
-    my $template = shift;
-
-    $self->log->debug( "Handling template " . $template );
-    eval { Jifty->handler->mason->handle_comp( $template ); };
-    my $err = $@;
-
-    # Handle parse errors
-    if ( $err and not eval { $err->isa('HTML::Mason::Exception::Abort') } ) {
-
-        # Save the request away, and redirect to an error page
-        Jifty->web->response->error($err);
-        my $c = Jifty::Continuation->new(
-            request  => Jifty->web->request->top_request,
-            response => Jifty->web->response,
-            parent   => Jifty->web->request->continuation,
-        );
-
-        warn "$err";
-
-        # Redirect with a continuation
-        Jifty->web->_redirect(
-            "/__jifty/error/mason_internal_error?J:C=" . $c->id );
-    }
-    elsif ($err) {
-        die $err;
-    }
-
-}
-
-
-=head2 import_plugins
-
-Imports rules from L<Jifty/plugins> into the main dispatcher's space.
-
-=cut
-
-sub import_plugins {
-    my $self = shift;
-
-    # Find the deferred rules
-    my @deferred;
-    push @deferred, $_->dispatcher->rules('DEFERRED') for Jifty->plugins;
-    push @deferred, $self->rules('DEFERRED');
-
-    # XXX TODO: Examine @deferred and find rles that cannot fire
-    # because they match no plugins; they should become un-deferred in
-    # the appropriate group.  This is so 'before plugin qr/Auth/' runs
-    # even if we have no auth plugin
-
-    for my $stage (qw/SETUP RUN CLEANUP/) {
-        my @groups;
-        push @groups, {name => ref $_,  rules => [$_->dispatcher->rules($stage)]} for Jifty->plugins;
-        push @groups, {name => 'Jifty', rules => [$self->rules($stage)]};
-
-        my @left;
-        my @rules;
-        for (@groups) {
-            my $name        = $_->{name};
-            my @group_rules = @{$_->{rules}};
-
-            # XXX TODO: 'after' rules should possibly be placed after
-            # the *last* thing they could match
-            push @rules, $self->_match_deferred(\@deferred, before => $name, $stage);
-            push @rules, @group_rules;
-            push @rules, $self->_match_deferred(\@deferred, after => $name, $stage);
-        }
-
-        no strict 'refs';
-        @{ $self . "::RULES_$stage" } = @rules;
-    }
-    if (@deferred) {
-        warn "Leftover unmatched deferred rules: ".Jifty::YAML::Dump(\@deferred);
-    }
-}
-
-sub _match_deferred {
-    my $self = shift;
-    my ($deferred, $time, $name, $stage) = @_;
-    my %stages = (SETUP => "before", RUN => "on", CLEANUP => "after");
-    $stage = $stages{$stage};
-
-    my @matches;
-    for my $op (@{$deferred}) {
-        # Only care if we're on the correct side of the correct plugin
-        next unless $op->[0] eq $time;
-
-        # Regex or string match, appropriately
-        next unless (
-            ref $op->[1]{plugin}
-            ? ( $name =~ $op->[1]{plugin} )
-            : ( $op->[1]{plugin} eq $name ) );
-
-        # Find the list of subrules
-        my @subrules = ref $op->[2] eq "ARRAY" ? @{$op->[2]} : ($op->[2]);
-
-        # Only toplevel rules make sense (before, after, on)
-        warn "Invalid subrule ".$_->[0] for grep {$_->[0] !~ /^(before|on|after)$/} @subrules;
-        @subrules = grep {$_->[0] =~ /^(before|on|after)$/} @subrules;
-
-        # Only match if the stage matches
-        push @matches, grep {$_->[0] eq $stage} @subrules;
-        @subrules = grep {$_->[0] ne $stage} @subrules;
-
-        $op->[2] = [@subrules];
-    }
-
-    # Clean out any completely matched rules
-    @$deferred = grep {@{$_->[2]}} @$deferred;
-
-    return @matches;
-}
-
-=head2 dump_rules
-
-Dump all defined rules in debug log. It will be called by Jifty on startup.
-
-=cut
-
-sub dump_rules {
-    my $self = shift;
-
-    no strict 'refs';
-    foreach my $stage ( qw/SETUP RUN CLEANUP/ ) {
-
-        my $log = '';
-        foreach my $r ( @{ $self . '::RULES_' . $stage } ) {
-            $log .= _unroll_dumpable_rules( 0,$r );
-        }
-
-        Jifty->log->debug( "Rules in stage $stage:\n", $log) if ($log);
-    }
-};
-
-=head2 _unroll_dumpable_rules LEVEL,RULE
-
-Walk all rules defined in dispatcher starting at rule
-C<RULE> and indentation level C<LEVEL>
-
-=cut
-
-sub _unroll_dumpable_rules {
-    my ($level, $rule) = @_;
-    my $log =
-        # indentation
-        ( "    " x $level ) .
-        # op
-        ( $rule->[0] || "undef op" ) . ' ' .
-        # arguments
-        (
-            ! defined( $rule->[1] )   ? ""                                          :
-            ref $rule->[1] eq 'ARRAY' ? "'" . join("','", @{ $rule->[1] }) . "'" :
-            ref $rule->[1] eq 'HASH'  ? $rule->[1]->{method} . " '" . $rule->[1]->{""} ."'" :
-            ref $rule->[1] eq 'CODE'  ? '{...}' :
-                                        "'" . $rule->[1] . "'"
-        ) .
-        "\n";
-
-    if (ref $rule->[2] eq 'ARRAY') {
-        $level++;
-        foreach my $sr ( @{ $rule->[2] } ) {
-            $log .= _unroll_dumpable_rules( $level, $sr );
-        }
-    }
-
-    return $log;
-}
 
 1;
